/**@file

// hostConfigIcemonHost.ctl
This library contains the function to configure the IcemonHost.

@par Creation Date
  dd/mm/yyyy

@par Modification History
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author 
  the author (DEP-GROUP)
*/

//@{

//------------------------------------------------------------------------------------------------------------------------
// hostConfigIcemonHost_checkConfig
/** check the device configuration
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dsConfigs input, config line as dyn_string
@param exceptionInfo output, for errors
*/
hostConfigIcemonHost_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  if (dynlen(dsConfigs) >= UN_CONFIG_COMMON_LENGTH)
  {
  // 1. Common
    unConfigGenericFunctions_checkParameters(dsConfigs, "IcemonHost", exceptionInfo);
  // 2. call the device check function
    unConfigGenericFunctions_checkDeviceConfig(dsConfigs, "IcemonHost", exceptionInfo);
  }
  else
  {
    fwException_raise(exceptionInfo,"ERROR","hostConfigConfigIcemonHost_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
  }
}

//------------------------------------------------------------------------------------------------------------------------
// hostConfigIcemonHost_setConfig
/** set the device configuration
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dsConfigs  input, config line as dyn_string
@param exceptionInfo output, for errors
*/
hostConfigIcemonHost_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sDpName;
  
// 1. Initialization : checking + dpCreate
  unConfigGenericFunctions_checkAll(dsConfigs, "IcemonHost", exceptionInfo);
  if (dynlen(exceptionInfo) <= 0)    // Good inputs
  {
    sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
    unConfigGenericFunctions_createDp(sDpName, "IcemonHost", exceptionInfo);
    if (dynlen(exceptionInfo) <= 0)  // Creation ok or dp already exists
    {
// 2. Generate device
// 2.1. Parameters
      unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
  // 2.2 call the device check function
      unConfigGenericFunctions_setDeviceConfig(dsConfigs, "IcemonHost", exceptionInfo);
    }    
  }
}

//------------------------------------------------------------------------------------------------------------------------
// IcemonHostsFE_IcemonHost_checkConfig
/** check the IcemonHost device configuration for the IcemonHostsFE front-end
  
@par Constraints
  None

@par Usage
  Internal

@par PVSS managers
  Ui, CTRL

@param dsConfigs input, config line as dyn_string
@param exceptionInfo output, for errors
*/
IcemonHostsFE_IcemonHost_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  bool bOk=false;
  
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host import IcemonHostsFE_IcemonHost", "IcemonHostsFE_IcemonHost_checkConfig", dsConfigs);
  if (dynlen(dsConfigs) == (HOST_CONFIG_ICEMONHOST_LENGTH + UN_CONFIG_COMMON_LENGTH))
    bOk = true;
  else if (dynlen(dsConfigs) == (HOST_CONFIG_ICEMONHOST_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
    bOk = true;
    unConfigGenericFunctions_checkArchive(dsConfigs[HOST_CONFIG_ICEMONHOST_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
                                          dsConfigs[HOST_CONFIG_ICEMONHOST_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
                                          dsConfigs[HOST_CONFIG_ICEMONHOST_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
                                          exceptionInfo);
  }
  else
    bOk = false;
  
  if(bOk) {
    // check all the device configuration
    // if the archive config is defined check if the archive config is Ok 
    //unConfigGenericFunctions_checkArchiveParameters(0, dsConfigs[UN_CONFIG_COMMON_LENGTH + HOST_CONFIG_ICEMONHOST_ARCHIVEACTIVE], exceptionInfo);
  }
  else
  {
    fwException_raise(exceptionInfo,"ERROR","IcemonHostsFE_IcemonHost_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
  }
}

//------------------------------------------------------------------------------------------------------------------------
// IcemonHostsFE_IcemonHost_setConfig
/** set the IcemonHost device configuration for the IcemonHostsFE front-end
  
@par Constraints
  None

@par Usage
  Internal

@par PVSS managers
  Ui, CTRL

@param dsConfigs input, config line as dyn_string
@param exceptionInfo output, for errors
*/
IcemonHostsFE_IcemonHost_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sDpName;
  bool bArchive;
  int iArchiveTimeFilter, iArchiveType, iSmoothProcedure;
  float fDeadband;
    
  sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host import IcemonHostsFE_IcemonHost "+sDpName, "IcemonHostsFE_IcemonHost_setConfig", dsConfigs);
// if archive name are in the config->overwrite panel setting.  
  if(dynlen(dsConfigs) == (HOST_CONFIG_ICEMONHOST_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
    if(dsConfigs[HOST_CONFIG_ICEMONHOST_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
      dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[HOST_CONFIG_ICEMONHOST_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];

    if(dsConfigs[HOST_CONFIG_ICEMONHOST_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
      dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[HOST_CONFIG_ICEMONHOST_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];

    if(dsConfigs[HOST_CONFIG_ICEMONHOST_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
      dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[HOST_CONFIG_ICEMONHOST_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
  }

  string hostname = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + HOST_CONFIG_ICEMONHOST_HOSTNAME_IDX];
  DebugN("Setting up host: " + hostname);
  dyn_string proxies = dpNames("FMC/" + hostname + "*");
  dpSet(sDpName + ".proxy", proxies);

  /* set all the device config that have to be set:
       -DPE alias: use unConfigGenericFunctions_setAlias
       -DPE description: use unConfigGenericFunctions_setDescription
       -periph address: use JCOP function
       -pv_range: use JCOP function
       -alarm:
       -archive:
// get archive config
  unConfigGenericFunctions_getArchiving(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + HOST_CONFIG_ICEMONHOST_ARCHIVEACTIVE], 
                                        bArchive, iArchiveTimeFilter,  iArchiveType, iSmoothProcedure);
  unConfigGenericFunctions_getArchivingDeadband(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + HOST_CONFIG_ICEMONHOST_ARCHIVEACTIVE], fDeadband);
// set archive config
  if(bArchive)
    fwArchive_set(sDpName + "...", dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL], 
                  iArchiveType, iSmoothProcedure, fDeadband, iArchiveTimeFilter,exceptionInfo);
  else
    fwArchive_delete(sDpName + ".ProcessOutput.value", exceptionInfo);
       -DPE value
       -proxy: create the proxy DP, set the proxy configuration, DPE config, etc.
       -set the faceplate limits for the DPE in faceplate: use unGenericDpFunctions_setFaceplateRange
       -etc.
  */
}

//------------------------------------------------------------------------------------------------------------------------
// IcemonHost_getArchiveDPE
/** return the list of IcemonHost DPE per archive (Bool, Ana, Event)
  
@par Constraints
  None

@par Usage
  Internal

@par PVSS managers
  Ui, CTRL

@return output, list of DPE
*/
/* Comment this function if you want to use the default setting: 
   Bool archive: bool, char DPE
   Ana archive: float DPE, int and uint DPE except evStsReg*
   Event archive: evStsReg* int and uint, struct DPE
*/
dyn_string IcemonHost_getArchiveDPE()
{
  dyn_string dsRet;
  dyn_string dsArchiveElements, exceptionInfo, dsDpel;
  dyn_int diDpelType;
  int i, len, j, k, length, lengthk, pos;
  string archiveBoolNames, archiveEventNames, archiveAnalogNames;
  string sDeviceType="IcemonHost";
  
//  dsRet[1] = ".ProcessInput.DPE1"+ ";"+".ProcessInput.DPE2";
//  dsRet[2] = archiveAnalogNames;
//  dsRet[3] = archiveEventNames;
  
  // the following code can also be used
  // in this example:
  // Bool archive: bool, char DPE
  // Ana archive: float DPE, blob DPE, all int DPE except evStsReg* 
  // Event archive: evStsReg* int, uint, struct DPE
     
  // get all the DPE having an archive config from the JCOP device definition
  fwDevice_getArchiveElements(sDeviceType, dsArchiveElements, exceptionInfo);
  // get all the DPE from the device type
  unGenericDpFunctions_getDPE(sDeviceType, dsDpel, diDpelType);
  
  length = dynlen(dsArchiveElements);
  lengthk = dynlen(dsDpel);
//DebugN(length, lengthk);
  for(j=1;j<=length;j++)
  {
    for(k=1;k<=lengthk;k++) {
//DebugN(dsDpel[k], dsArchiveElements[j]);
      if((sDeviceType+dsArchiveElements[j]) == dsDpel[k]) {
        switch(diDpelType[k]){
          case DPEL_CHAR:
          case DPEL_BOOL:
            archiveBoolNames = archiveBoolNames + dsArchiveElements[j] + ";";
            break;
          case DPEL_FLOAT:
            archiveAnalogNames = archiveAnalogNames + dsArchiveElements[j] + ";";
            break;
          case DPEL_INT:
            // if DPE like evStsReg --> put it in event archive
            if (patternMatch(".ProcessInput.evStsReg*", dsArchiveElements[j]))
            {
              archiveEventNames = archiveEventNames + dsArchiveElements[j] + ";";
            }
            else
            {
              archiveAnalogNames = archiveAnalogNames + dsArchiveElements[j] + ";";
            }
            break;
          case DPEL_UINT:
          case DPEL_BOOL_STRUCT:
            archiveEventNames = archiveEventNames + dsArchiveElements[j] + ";";
            break;
          case DPEL_BLOB:
            archiveAnalogNames = archiveAnalogNames + dsArchiveElements[j] + ";";
            break;
        }
      }
    }
  }
  dsRet[1] = archiveBoolNames;
  dsRet[2] = archiveAnalogNames;
  dsRet[3] = archiveEventNames;
  
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host import IcemonHost", "IcemonHost_getArchiveDPE", dsRet);
  return dsRet;
}

//------------------------------------------------------------------------------------------------------------------------
// IcemonHost_getArchiveProxyDPE
/** return a dyn_dyn_string with the list of proxy DPE linked (or that will be linked) 
to the device for the three archives: boolean, analog and event

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dsConfigLine input, the device configuration
@return the list of DPE per archive (3)
*/
dyn_dyn_string IcemonHost_getArchiveProxyDPE(dyn_string dsConfigLine)
{
  dyn_dyn_string dds;
  
  //return here for the boolean archive the list of proxy DPEs linked to the device
  dds[UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] = makeDynString();
  //return here for the analog archive the list of proxy DPEs linked to the device
  dds[UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] = makeDynString();
  //return here for the event archive the list of proxy DPEs linked to the device
  dds[UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] = makeDynString();
  
  return dds;
}

//------------------------------------------------------------------------------------------------------------------------

//@}
