/**@file

// hostExportDevices.ctl
This library contains the export functions of the package.

@par Creation Date
  dd/mm/yyyy

@par Modification History
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author 
  the author (DEP-GROUP)
*/

//@{

//------------------------------------------------------------------------------------------------------------------------
// IcemonHostsFE_Com_ExportConfig
/** check the content of the delete command
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dsParam  input, parameters from the export panel (FE, application, device type)
@param exceptionInfo output, for errors
*/
IcemonHostsFE_Com_ExportConfig(dyn_string dsParam, dyn_string &exceptionInfo)
{
  int iRes;
  string sPlcName, sPlcDrv;
  string sType, sDp, sCounterAddress;
  dyn_string dsDpParameters;
  dyn_string dsAnalogArchive, dsEventArchive, dsBoolArchive, dsSubApplications;
  dyn_string exInfo;
  int pos;

  if (dynlen(dsParam)==UN_CONFIG_EXPORT_LENGTH)
  {
    sPlcName = dsParam[UN_CONFIG_EXPORT_PLCNAME];
    sDp = c_hostSystemIntegrity_ICEMONHOSTSFE + sPlcName;
    iRes = dpGet(sDp + ".configuration.subApplications", dsSubApplications,
                  sDp + ".configuration.archive_bool", dsBoolArchive,
                  sDp + ".configuration.archive_analog", dsAnalogArchive,
                  sDp + ".configuration.archive_event", dsEventArchive,
                  sDp + ".communication.driver_num", sPlcDrv);
    TextFieldDriver.text = sPlcDrv;
  
    if (dynlen(dsBoolArchive)>0 && dynlen(dsAnalogArchive)>0 && dynlen(dsEventArchive)>0)
    {
      pos = dynContains(dsSubApplications, dsParam[UN_CONFIG_EXPORT_APPLICATION]);
      if(pos >=1 ) 
      {
        TextFieldBoolArchive.text = dsBoolArchive[pos];
        TextFieldAnalogArchive.text = dsAnalogArchive[pos];
        TextFieldEventArchive.text = dsEventArchive[pos];
      }
      else
      {
        TextFieldBoolArchive.text = dsBoolArchive[1];
        TextFieldAnalogArchive.text = dsAnalogArchive[1];
        TextFieldEventArchive.text = dsEventArchive[1];
      }
    }

    //Delete
    dynClear(dsDpParameters);
    hostExportDevice_writeDeviceTypeLineFormat(HOST_ICEMONHOSTSFE_DPTYPE, UN_DELETE_COMMAND);
    dynAppend(dsDpParameters, "#" + UN_DELETE_COMMAND);
    dynAppend(dsDpParameters, dsParam[UN_CONFIG_EXPORT_PLCNAME]);
    dynAppend(dsDpParameters, dsParam[UN_CONFIG_EXPORT_APPLICATION]);
    unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
  
    //PLCCONFIG
    hostExportDevice_writeDeviceTypeLineFormat(HOST_ICEMONHOSTSFE_DPTYPE, UN_PLC_COMMAND);
    dynClear(dsDpParameters);
    dynAppend(dsDpParameters, UN_PLC_COMMAND);
    dynAppend(dsDpParameters, HOST_ICEMONHOSTSFE_DPTYPE);
    dynAppend(dsDpParameters, dsParam[UN_CONFIG_EXPORT_PLCNAME]);
    dynAppend(dsDpParameters, dsParam[UN_CONFIG_EXPORT_APPLICATION]);

    // get all the IcemonHostsFE configuration
    // dpGet(...);
    
    // write to file
    unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);

 // get the front-end system alarm
/*    fputs("#" + "\n" , g_fileToExport);
    dsAlarmDps = dpNames(c_unSystemAlarm_dpPattern + FE_pattern + c_hostSystemIntegrity_ICEMONHOSTSFE + dsParam[UN_CONFIG_EXPORT_PLCNAME] + "_*", c_unSystemAlarm_dpType);

    iLen=dynlen(dsAlarmDps);
    if(iLen > 0) {
      hostExportDevice_writeDeviceTypeLineFormat(UN_FESYSTEMALARM_COMMAND, "");
    }
    for(i=1; i<=iLen; i++)
    {
      if (dpExists(dsAlarmDps[i]))
      {
        sSplitModPlc = substr(dsAlarmDps[i], strpos(dsAlarmDps[i], dsParam[UN_CONFIG_EXPORT_PLCNAME]) + strlen(dsParam[UN_CONFIG_EXPORT_PLCNAME]) + 1);
        
// get the number (stored in the description)

        sNbr = unGenericDpFunctions_getDescription(dsAlarmDps[i]+".");
        unConfigGenericFunctions_checkUnsigned(sNbr, uNbr, exInfo);
//DebugN(sNbr, uNbr);
        if(uNbr > 0) {
          //Get Alarm Address
          dpGet(dsAlarmDps[i] + ".alarm:_address.._drv_ident" , sAlarmAddress);
          hostConfigGenericFunctions_getAddressConfig(dsAlarmDps[i] + ".alarm", dsAddressDpParameters);

          //Get alarm type
          dpGet(dsAlarmDps[i] + ".alarm:_alert_hdl.._type", sAlarmType);
          if(sAlarmType == DPCONFIG_ALERT_NONBINARYSIGNAL)
          {
            fwAlertConfig_getLimits(dsAlarmDps[i] + ".alarm", limitNumbers, alertLimits, exceptionInfo);
            alertRanges = (dynMax(limitNumbers) + 1);
            dsAlertText = makeDynString();
            for(j=1; j<=alertRanges; j++)
            {
              dpGet(dsAlarmDps[i] + ".alarm:_alert_hdl." + j + "._text", sAlertText);
              dynAppend(dsAlertText, sAlertText);
            }
                                        
  //DebugN(dsAlertText);
            if(dynlen(dsAlertText) > 0)
            {
              if (dsAlertText[1]==UN_SYSTEMINTEGRITY_OK)
              {
                sType = "TRUE";
              }
              else
              {
                sType = "FALSE";
              }
            }
            else
              sType = UN_CONFIG_EXPORT_UNDEF;
          }
          else
            sType = UN_CONFIG_EXPORT_UNDEF;  
  
          dynClear(dsDpParameters);
          dynAppend(dsDpParameters, UN_FESYSTEMALARM_COMMAND);
          dynAppend(dsDpParameters, sNbr);                                                              //???? nbr
          dynAppend(dsDpParameters, sSplitModPlc);                                                  //identifier
          dynAppend(dsDpParameters, sNbr);                                                              //???? nbr
          dynAppend(dsDpParameters, sType);                                                          //alarm type
          dynAppend(dsDpParameters, unGenericDpFunctions_getDescription(dsAlarmDps[i] + ".alarm"));  //description
          dynAppend(dsDpParameters, dsAddressDpParameters);                                                  //address
  
          fputs("#For IcemonHostsFE " + sNbr + "\n" , g_fileToExport);
  // write to file
          unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
        }
      }
    }
    */
  }
  else
    fwException_raise(exceptionInfo, "ERROR", "IcemonHostsFE_Com_ExportConfig(): Wrong Parameters", "");    
}

//------------------------------------------------------------------------------------------------------------------------
// hostExportDevice_writeDeviceTypeLineFormat
/** returns the format of the config line for a device type
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDeviceType input, device type
@param sCommand input, keyword (for front-end only)
*/
hostExportDevice_writeDeviceTypeLineFormat(string sDeviceType, string sCommand)
{
  string sFormat, sMandatory, sDeviceConfig;
  
  sMandatory = sDeviceType+";Device number (unique);Device name (Device Identifier);description;Diagnostic panel;HTML page;Default panel;subsystem1 (Domain);subsystem2 (Nature);Widget Name";
  switch(sDeviceType) 
  {
    case HOST_ICEMONHOSTSFE_DPTYPE:
      switch(sCommand)
      {
        case UN_PLC_COMMAND:
          sFormat = UN_PLC_COMMAND+";"+HOST_ICEMONHOSTSFE_DPTYPE+";Front-end device name;Front-end application;";
          sDeviceConfig = "";
          fputs("#format: " +sFormat + sDeviceConfig + "\n" , g_fileToExport);
          break;
        case UN_DELETE_COMMAND:
          sFormat =  UN_DELETE_COMMAND+";Front-end device name;Front-end application (or *);[Device type;[Device id;]]";
          fputs("#format: " +sFormat + "\n" , g_fileToExport);
          break;
        default:
          break;
      }
      break;
    case UN_FESYSTEMALARM_COMMAND:
//        sFormat = UN_FESYSTEMALARM_COMMAND+";WorldFip number;Device Name (device Identifier);WorldFip number;alarm type (FALSE=alarm if value<10/TRUE=alarm if value>10);description;CMW Device;CMW property;CMW tag;CMW context filter;";
        fputs("#format: " +sFormat + "\n" , g_fileToExport);
        break;
    case "IcemonHost":
      sDeviceConfig =   "";
      sFormat = sMandatory+";"+sDeviceConfig;
      fputs("#format: " +sFormat + "\n" , g_fileToExport);
      break;
    default:
      fputs("#no format: \n" , g_fileToExport);
      break;
  }
}

//------------------------------------------------------------------------------------------------------------------------
// IcemonHostsFE_IcemonHost_ExportConfig
/** returns the format of the config line for a device type
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dsDpList input, list of device dp
@param exceptionInfo output, the errors are returned here
*/
void IcemonHostsFE_IcemonHost_ExportConfig(dyn_string dsDpList, dyn_string &exceptionInfo)
{
  dyn_string dsAllCommonParameters, dsDpParameters, dsArchiveNames, dsAddressDpParameters;
  string sDeviceType, sArchiveActive, sArchiveName;
  float fArchiveTimeFilter;
  int i,iLeni, j, iLenj;
    
  sDeviceType="IcemonHost";
  
  iLeni=dynlen(dsDpList);
  if(iLeni > 0) {
    hostExportDevice_writeDeviceTypeLineFormat(sDeviceType, "");
  }
  for(i=1; i<=iLeni; i++)
  {                  
    if (dpExists(dsDpList[i]))
    {
      dynClear(dsAllCommonParameters);
      dynClear(dsDpParameters);
      dynClear(dsArchiveNames);
        
      //Get all common Parameters
      unExportDevice_getAllCommonParameters(dsDpList[i], sDeviceType ,  dsAllCommonParameters);
      dsDpParameters = dsAllCommonParameters;
          
      //Get the device configuration
	  /*
	  - archive config
      unExportDevice_getArchiveParametersDpe(dsDpList[i]+"...", sArchiveActive, fArchiveTimeFilter, "");
	  - address config
	  fwPeriphAddress_get(dsDpList[i] + "...", bCPUExist, daCPUCounterConfig, bIsActive, exceptionInfo);
	  - smoothing config
	  unExportDevice_fwSmoothing_getTemp(dsDpList[i] + "...", bIsSmoothDefined, iSmoothProcedure, fDeadbandTemp, fTimeInterval);
	  unExportDevice_getDeadBand(fDeadbandTemp, bIsSmoothDefined, sMax, sMin, iSmoothProcedure, fDeadband, iDeadBandType);
	  - pv_range
	  fwPvRange_get(dsDpList[i] + "...", bDoesExist , fMinValue , fMaxValue , bNegateRange , bIgnoreOutside , bInclusiveMin , bInclusiveMax , exceptionInfo);
	  - etc.
	  */
	  
      //Get Archive Names for HOST devices
      hostConfigGenericFunctions_getArchiveNames(dsDpList[i], sDeviceType, dsArchiveNames);
      iLenj=dynlen(dsArchiveNames);
      for(j=1; j<=iLenj ; j++)
      {
        dynAppend(dsDpParameters, dsArchiveNames[j]);
      }  
      // write the data to the file
      unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
    }//dp doesn't exist
  }//end for
}

//------------------------------------------------------------------------------------------------------------------------

//@}
