#uses "fwFMC/fwFMC.ctl"

/**@file

// hostIcemonHost.ctl
This library contains the widget, faceplate, etc. functions of IcemonHost.

@par Creation Date
  dd/mm/yyyy

@par Modification History
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author 
  the author (DEP-GROUP)
*/

//@{

//------------------------------------------------------------------------------------------------------------------------
// hostIcemonHost_AcknowledgeAlarm
/** Returns the list of IcemonHost DPE with alarm config that can be acknowledged
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpName  input, device name DP name
@param sDpType input, device type
@param dsNeedAck output, the lsit of DPE
*/
hostIcemonHost_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
  dsNeedAck = makeDynString("ProcessInput....");
  // if the acknoledge must be sent to the PLC: CPC Front-end: add UN_ACKNOWLEDGE_PLC
  //dsNeedAck = makeDynString("ProcessInput....", UN_ACKNOWLEDGE_PLC);
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host AcknowledgeAlarm IcemonHost "+sDpName, "hostIcemonHost_AcknowledgeAlarm", sDpName, sDpType, dsNeedAck);
}

//------------------------------------------------------------------------------------------------------------------------
// hostIcemonHost_ObjectListGetValueTime
/** Function called from snapshot utility of the treeDeviceOverview to get the time and value
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDeviceName  input, device name
@param sDeviceType input, device type
@param dsReturnData output, return data, array of 5 strings
*/
hostIcemonHost_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
  string sTime, sValue, sState, sColor;
  bool bOk, bBad;
  
  dpGet(sDeviceName+".ProcessInput....", sValue, 
        sDeviceName+".ProcessInput....", sState, 
        sDeviceName+".ProcessInput....:_online.._stime", sTime, 
        sDeviceName+".ProcessInput....:_online.._bad", bBad);

  dsReturnData[1] = sTime;
  dsReturnData[2] = sValue;
  dsReturnData[3] = bBad;
  dsReturnData[4] = sState;
  dsReturnData[5] = sColor;
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host snapshot IcemonHost "+sDeviceName, "hostIcemonHost_ObjectListGetValueTime", sDeviceName, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_FaceplateStatusRegisterCB
/** faceplate DistributedControl callback of the faceplate status panel
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDp input, the DistributedControl system name DP name
@param bSystemConnected input, the state of the system name
*/
hostIcemonHost_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
  string deviceName, sSystemName;
  int iAction, iRes;
  dyn_string exceptionInfo;
  bool bRemote;
  
  deviceName = unGenericDpFunctions_getDpName($sDpName);
  sSystemName = unGenericDpFunctions_getSystemName(deviceName);
  unDistributedControl_isRemote(bRemote, sSystemName);
  if(bRemote)
    g_bSystemConnected = bSystemConnected;
  else
    g_bSystemConnected = true;
                                         
  unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host faceplate IcemonHost "+sDp, "hostIcemonHost_FaceplateStatusRegisterCB", sDp, deviceName, g_bCallbackConnected, iAction);
  switch (iAction)
  {
    case UN_ACTION_DISCONNECT:
      hostIcemonHost_FaceplateStatusDisconnection();
      break;
    case UN_ACTION_DPCONNECT:
      hostIcemonHost_FaceplateConnect(deviceName);
      break;
    case UN_ACTION_DPDISCONNECT_DISCONNECT:
      hostIcemonHost_FaceplateDisconnect(deviceName);
      hostIcemonHost_FaceplateStatusDisconnection();
      break;
    case UN_ACTION_NOTHING:
    default:
      break;
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_FaceplateStatusDisconnection
/** set the faceplate when the device system is disconnected 
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

*/
hostIcemonHost_FaceplateStatusDisconnection()
{
// 2. animate FE Connection
  hostGenericFunctions_FaceplateStatusDisconnect();
  // there is no need to do here execScript because there is one faceplate per device type
  // set all the graphical element to the state data not connnected
  // ex. of functions that can be used
  //unGenericObject_ColorBoxDisconnect, unGenericObject_DisplayValueDisconnect, etc.
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host faceplate IcemonHost", "hostIcemonHost_FaceplateStatusDisconnection");
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_FaceplateConnect
/** dpConnect to the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
*/
hostIcemonHost_FaceplateConnect(string deviceName)
{
  int iAction, iRes;
  dyn_string exceptionInfo;
  string sFrontEnd, sSystemName;
  
  g_sDeviceAlias = unGenericDpFunctions_getAlias(deviceName);

  g_bUnSystemAlarmPlc = false;
// get the systemName
  sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
  sFrontEnd = unGenericObject_GetPLCNameFromDpName(deviceName);
  g_sFrontEndDp = sSystemName+sFrontEnd;
  g_sFrontEndAlias = unGenericDpFunctions_getAlias(g_sFrontEndDp);
  
  if(sFrontEnd != ""){
    sFrontEnd = sSystemName+c_unSystemAlarm_dpPattern+DS_pattern+sFrontEnd;
//DebugN(sSystemName, sFrontEnd);
    if(dpExists(sFrontEnd)) {
      g_bUnSystemAlarmPlc = true;
    }
  }
    
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host faceplate IcemonHost "+deviceName, "hostIcemonHost_FaceplateConnect", deviceName, sFrontEnd, g_bUnSystemAlarmPlc);
  if(g_bUnSystemAlarmPlc) {    
    // set all faceplate global variable usefull for the callback, e.g.: description, format, unit, etc.
    // set graphical element of the faceplate to the init state if needed
    
    // if there are many dpConnect possibilities: e.g. dpConnect with or without _alert_hdl config
    // keep the kind of dpConnect in a variable global to the faceplate and do a swith for the dpConnect
    // and do to the correct dpDisconnect in the hostIcemonHost_FaceplateDisconnect
    iRes = dpConnect("hostIcemonHost_FaceplateStatusAnimationCB",
                      deviceName + ".ProcessInput.data",
                      deviceName + ".ProcessInput.data:_online.._invalid",
                      deviceName + ".ProcessInput.iData",
                      sFrontEnd+".alarm",
                      sFrontEnd+".enabled");
    g_bCallbackConnected = (iRes >= 0);
  }
  else
    hostIcemonHost_FaceplateStatusDisconnection();
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_FaceplateDisconnect
/** dpDisconnect to the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
*/
hostIcemonHost_FaceplateDisconnect(string deviceName)
{
  int iRes;
  string sFrontEnd, sSystemName;
  
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host faceplate IcemonHost "+deviceName, "hostIcemonHost_FaceplateDisconnect", deviceName, g_bUnSystemAlarmPlc);
  if(g_bUnSystemAlarmPlc) {
// get the systemName
    sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
    sFrontEnd = unGenericObject_GetPLCNameFromDpName(deviceName);
    sFrontEnd = sSystemName+c_unSystemAlarm_dpPattern+DS_pattern+sFrontEnd;

    // if there were many dpConnect possibilities: e.g. dpConnect with or without _alert_hdl config
    // re-use the kind of dpConnect kept in a variable global to the faceplate set during hostIcemonHost_FaceplateConnect
    // and do to the correct dpDisconnect
    iRes = dpDisconnect("hostIcemonHost_FaceplateStatusAnimationCB",
                      deviceName + ".ProcessInput.data",
                      deviceName + ".ProcessInput.data:_online.._invalid",
                      deviceName + ".ProcessInput.iData",
                      sFrontEnd+".alarm",
                      sFrontEnd+".enabled");
    g_bCallbackConnected = !(iRes >= 0);
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_FaceplateStatusAnimationCB
/** callback function on the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDp1 input, the DPE1 
@param blobData input, device data
@param sDpInvalid input, the invalid DPE 
@param bInvalid input, data validity
@param sDp2 input, the DPE2 
@param iData input, device status data
@param sDpFESystemIntegrityAlarmValue input, front-end device system integrity alarm DPE 
@param iFESystemIntegrityAlarmValue input, front-end device system integrity alarm value
@param sDpFESystemIntegrityAlarmEnabled input, front-end device system integrity enable DPE
@param bFESystemIntegrityAlarmEnabled input, front-end device system integrity enable value
*/
hostIcemonHost_FaceplateStatusAnimationCB(string sDp1, blob blobData, 
                                                    string sDpInvalid, bool bInvalid,
                                                    string sDp2, int iData,
                                                    string sDpFESystemIntegrityAlarmValue, int iFESystemIntegrityAlarmValue, 
                                                    string sDpFESystemIntegrityAlarmEnabled, bool bFESystemIntegrityAlarmEnabled)
{
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host faceplate IcemonHost "+sDp1, "hostIcemonHost_FaceplateStatusAnimationCB", sDp1, bInvalid, blobData, iData, g_bSystemConnected, 
                               iFESystemIntegrityAlarmValue, bFESystemIntegrityAlarmEnabled);
  if(g_bSystemConnected)
    hostGenericFunctions_FaceplateStatusHeader(g_sFrontEndDp, g_sFrontEndAlias, g_sDeviceAlias, iFESystemIntegrityAlarmValue, 
                                              bFESystemIntegrityAlarmEnabled, bInvalid);
  // animate all the graphical element of the faceplate
  // all the variables global to the faceplate can be used
  // encapsulate all the setValue or function call with the following if(g_bSystemConnected)
  /* e.g.:
    if(g_bSystemConnected)
      unGenericObject_DisplayValue(...);
    */
  // ex. of functions that can be used: unGenericObject_DisplayValue, unGenericObject_ColorBoxAnimate, 
  // unGenericObject_ColorBoxAnimateAlarm, unGenericObject_ColoredSquareAnimate, 
  // unGenericObject_DisplayRange, etc.
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_ButtonRegisterCB
/** Contextual button DistributedControl callback of the contextual device button panel
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDp input, the DistributedControl system name DP name
@param bSystemConnected input, the state of the system name
*/
hostIcemonHost_ButtonRegisterCB(string sDp, bool bSystemConnected)
{
  string deviceName, sDpType, sFunction;
  int iAction, iRes, iManReg01;
  dyn_string exceptionInfo, dsFunctions, dsUserAccess;
  
  deviceName = unGenericDpFunctions_getDpName($sDpName);
  unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host button IcemonHost "+deviceName, "hostIcemonHost_ButtonRegisterCB", sDp, deviceName, g_bCallbackConnected, iAction);
  switch (iAction)
  {
    case UN_ACTION_DISCONNECT:
      hostIcemonHost_ButtonDisconnection();
      break;
    case UN_ACTION_DPCONNECT:

      sDpType="IcemonHost";
      unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
      sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
             
      if (sFunction != "")
      {
        evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" + 
               "dyn_string dsUserAccessReturn;" +
               "if (isFunctionDefined(\"" + sFunction  + "\"))" +
               "    {" +
                 sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
               "    }" +
               "return dsUserAccessReturn; }", makeDynString(), deviceName, sDpType);
        // keep the list of the device allowed action 
        g_dsUserAccess=dsUserAccess;  
      }
      else
        fwException_raise(exceptionInfo,"ERROR", "hostIcemonHost_ButtonRegisterCB:" + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");

    // set all contextual panel global variable usefull for the hostIcemonHost_ButtonAnimationCB, e.g.: description, format, unit, etc.
      unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host button IcemonHost "+deviceName, "hostIcemonHost_ButtonRegisterCB Connect", deviceName, sDpType, dsUserAccess);
      iRes = dpConnect("hostIcemonHost_ButtonAnimationCB",
               deviceName + ".statusInformation.selectedManager:_lock._original._locked",
               deviceName + ".statusInformation.selectedManager",
               deviceName + ".ProcessInput.data",
               deviceName + ".ProcessInput.iData",
               deviceName + ".ProcessInput.data:_online.._invalid");
      g_bCallbackConnected = (iRes >= 0);
      break;
    case UN_ACTION_DPDISCONNECT_DISCONNECT:
      unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host button IcemonHost "+deviceName, "hostIcemonHost_ButtonRegisterCB Disconnect", deviceName);
      iRes = dpDisconnect("hostIcemonHost_ButtonAnimationCB",
               deviceName + ".statusInformation.selectedManager:_lock._original._locked",
               deviceName + ".statusInformation.selectedManager",
               deviceName + ".ProcessInput.data",
               deviceName + ".ProcessInput.iData",
               deviceName + ".ProcessInput.data:_online.._invalid");
      g_bCallbackConnected = !(iRes >= 0);
      hostIcemonHost_ButtonDisconnection();
      break;
    case UN_ACTION_NOTHING:
    default:
      break;
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_UserLoginGetButtonState
/** return the device data, this function is called when the user logs in and logs out
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
@param sType input, the device type
@param dsReturnData output, the device data, [1] = lock state, [2] = lock by, [3] .. [6] device data
*/
hostIcemonHost_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
  int iRes;
  string sDpLock, sDpSelectedManager, sSelectedManager;
  bool bLocked;

  // read the device data (6 string)
  sDpLock = deviceName + ".statusInformation.selectedManager:_lock._original._locked";
  sDpSelectedManager = deviceName + ".statusInformation.selectedManager";
  if (dpExists(deviceName))
  {
    iRes = dpGet(sDpLock, bLocked, sDpSelectedManager, sSelectedManager);
  }
  else
  {
    iRes = -1;
  }
  if (iRes < 0) // Errors during dpget
  {
    bLocked = false;
    sSelectedManager = "";
  }
  dsReturnData=makeDynString(bLocked, sSelectedManager, "..", "..", "..", "..");
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host button IcemonHost "+deviceName, "hostIcemonHost_UserLoginGetButtonState", deviceName, sType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_ButtonSetState
/** Set the state of the contextual button of the device
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
@param sDpType input, the device type
@param dsUserAccess input, list of allowed action on the device
@param dsData input, the device data [1] = lock state, [2] = lock by, [3] .. [6] device data
*/
hostIcemonHost_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
  bool bSelected;
  dyn_string dsButtons;
  int i, length;
  string localManager;
  bool bLocked, buttonEnabled;
  string sSelectedManager, sDeviceValue1, sDeviceValue2, sDeviceValue3, sDeviceValue4;

  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host button IcemonHost "+deviceName, "hostIcemonHost_ButtonSetState", deviceName, sDpType, dsUserAccess, dsData);
  bLocked = dsData[1];
  sSelectedManager = dsData[2];
  sDeviceValue1 = dsData[3];
  sDeviceValue2 = dsData[4];
  sDeviceValue3 = dsData[5];
  sDeviceValue4 = dsData[6];

  // dsButtons = list of all the possible actions except Select 
  // use the name of the graphical element without "button"
  // use the name of the graphical elements.
  dsButtons = makeDynString("Cmd1","..."); 
  length = dynlen(dsButtons);
// 1. Selection state
  localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
  bSelected = (localManager == "S");
// 2. for each possible action check if the action is in the list dsUserAccess
  // crosscheck if needed with the state of the device
  for(i=1;i<=length;i++)
  {
    buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);  // User access
    switch(dsButtons[i])
    {
      case "..":
        // calculate the enable state of the button from the device data
        buttonEnabled = buttonEnabled & bSelected & (sDeviceValue1 == "..");
        break;
      case "..":
        break;
      default:
        break;
    }
    setValue(UN_FACEPLATE_BUTTON_PREFIX + dsButtons[i], "enabled", buttonEnabled);
  }
// 3. Button Select state
  unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_ButtonDisconnection
/** set the state of the device action button when the device system is disconnected 
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

*/
hostIcemonHost_ButtonDisconnection()
{
  // set all the device action button to disabled
  unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_SELECT, "Cmd1", ".."));
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host button IcemonHost", "hostIcemonHost_ButtonDisconnection");
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_ButtonUserAccess
/** returns the list of allowed action on the device for a user logged in
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpName input, the device DP name
@param sDpType input, the device type
@param dsAccess input, list of allowed action on the device
*/
hostIcemonHost_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
  bool operator, expert, admin;
  int i, length;
  dyn_bool dbPermissions;
  dyn_string exceptionInfo;
  string sOperAction, sExpAction, sAdminAction;
  bool bActionDefined = false;

  // get the configured list of authorized action per privilege
  unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
  if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
     (sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
    bActionDefined=true;
  operator = dbPermissions[2];
  expert = dbPermissions[3];
  admin = dbPermissions[4];
  dsAccess = makeDynString();
  
  // for each privilege get the list of allowed actions if there are action defined on the device
  // if there are no action defined for the device get the default one
  // in the example, the default behavior is 
  // the Select and Cmd1 buttons are allowed if one has the expert privilege
  if (operator)
  {
    if(bActionDefined)
      dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
    else
      dynAppend(dsAccess, makeDynString(".."));
  }
  if (expert)
  {
    if(bActionDefined)
      dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
    else 
      dynAppend(dsAccess, makeDynString("Cmd1", UN_FACEPLATE_BUTTON_SELECT, ".."));
  }
  if(admin){
    if(bActionDefined)
      dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
    else
      dynAppend(dsAccess, makeDynString(".."));
  }
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host button IcemonHost "+sDpName, "hostIcemonHost_ButtonUserAccess", sDpName, sDpType, dbPermissions, sOperAction, sExpAction, sAdminAction, dsAccess);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_ButtonAnimationCB
/** callback function on the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpLock input, the lock DPE 
@param bLocked input, selected lock state of the device
@param sDpSelect input, the select DPE 
@param sSelectedManager input, user and Ui that selected the device
@param sDp1 input, the DPE1 
@param blobData input, device data
@param sDp2 input, the DPE2 
@param iData input, device status data
@param sDpInvalid input, the invalid DPE 
@param bInvalid input, data validity
*/
hostIcemonHost_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager,
                                           string sDp1, blob blobData, string sDp2, int iData,
                                           string sDpInvalid, bool bInvalid)
{
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host button IcemonHost "+sDp1, "hostIcemonHost_ButtonAnimationCB", sDp1, blobData, iData, bInvalid, g_dsUserAccess);
  // call the set button state with the device data and the current allowed action on the device
  // which is in g_dsUserAccess
  hostIcemonHost_ButtonSetState(unGenericDpFunctions_getDpName(sDp1), "IcemonHost", g_dsUserAccess, 
                  makeDynString(bLocked, sSelectedManager, "..", "..", "..", ".."));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_WidgetRegisterCB
/** widget DistributedControl callback
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDp input, the DistributedControl system name DP name
@param bSystemConnected input, the state of the system name
*/
hostIcemonHost_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
  string deviceName, sSystemName;
  int iAction;
  dyn_string exceptionInfo;
  string sFrontEnd;
  bool bRemote;
  
  deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
  if (deviceName == "")  // In case of disconnection
  {
    deviceName = g_sDpName;
  }
  g_bUnSystemAlarmPlc = false;
// get the systemName
  sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
  sFrontEnd = unGenericObject_GetPLCNameFromDpName(deviceName);

  unDistributedControl_isRemote(bRemote, sSystemName);
  if(bRemote)
    g_bSystemConnected = bSystemConnected;
  else
    g_bSystemConnected = true;

  if(sFrontEnd != ""){
    sFrontEnd = sSystemName+c_unSystemAlarm_dpPattern+DS_pattern+sFrontEnd;
//DebugN(sSystemName, sFrontEnd);
    if(dpExists(sFrontEnd)) {
      g_bUnSystemAlarmPlc = true;
    }
  }

  unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host widget IcemonHost "+deviceName, "hostIcemonHost_WidgetRegisterCB", sDp, deviceName, g_bCallbackConnected, iAction);

  //Resolve FSM DP:
  string fsmDp = "";
  if(isDollarDefined("$1")&&isDollarDefined("$2") && $1 != "" && $2 != "")
  {
    fsmDp = getSystemName() + $1 + "|" + $2;
  }
  else
  {
    string fmcNode = unGenericDpFunctions_getAlias(deviceName);
    dyn_string dps = dpNames("*" + fmcNode, "_FwFsmDevice");
    if(dynlen(dps)) fsmDp = dps[1];
DebugN(fmcNode, fsmDp);
  }
  

  switch(iAction)
  {
    case UN_ACTION_DISCONNECT:
      hostIcemonHost_WidgetDisconnection(g_sWidgetType);
      break;
    case UN_ACTION_DPCONNECT:
      g_sDpName = deviceName;
      hostIcemonHost_WidgetConnect(deviceName, sFrontEnd, fsmDp);
      break;
    case UN_ACTION_DPDISCONNECT_DISCONNECT:
      hostIcemonHost_WidgetDisconnect(deviceName, sFrontEnd, fsmDp);
      hostIcemonHost_WidgetDisconnection(g_sWidgetType);
      break;
    case UN_ACTION_NOTHING:
    default:
      break;
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_WidgetDisconnection
/** set the widget when the system is disconnected 
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sWidgetType input, the type of widget
*/
hostIcemonHost_WidgetDisconnection(string sWidgetType)
{
  /** use the exec mechanism to allow multiple widget. **/
  string sFunction = "hostIcemonHost_"+sWidgetType+"Disconnection";

  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host widget IcemonHost", "hostIcemonHost_WidgetDisconnection", sWidgetType);
  if (isFunctionDefined(sFunction) && (sFunction != ""))
  {
    execScript("main(string sWidgetType) {" + sFunction + "(sWidgetType);}", makeDynString(), sWidgetType);
  }
}
//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_WidgetConnect
/** dpConnect to the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, front-end device DP name
@param sFrontEnd input, front-end device name
*/
hostIcemonHost_WidgetConnect(string deviceName, string sFrontEnd, string fsmDp)
{
  int iRes;
  string sSystemName;
  
//  check if the device name is of type "IcemonHost" if not like no front-end. -> not available.
  if(dpTypeName(deviceName) != "IcemonHost")
    g_bUnSystemAlarmPlc = false;
  
  sSystemName = unGenericDpFunctions_getSystemName(deviceName);
  
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host widget IcemonHost "+deviceName, "hostIcemonHost_WidgetConnect", deviceName, g_bUnSystemAlarmPlc);
  if(g_bUnSystemAlarmPlc)
  {
    // set all widget global variable usefull for the callback, e.g.: description, format, unit, etc.
    
    // if there are many dpConnect possibilities: e.g. dpConnect with or without _alert_hdl config
    // keep the kind of dpConnect in a variable global to the widget and do a swith for the dpConnect
    // and do to the correct dpDisconnect in the hostIcemonHost_WidgetDisconnect
    dyn_string proxyDeviceDps;
    unGenericDpFunctions_getProxyDeviceList(deviceName, iRes, proxyDeviceDps);
    iRes = dpConnect("hostIcemonHost_WidgetCB", 
                          deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                          deviceName + ".statusInformation.selectedManager",
                          fsmDp + ".fsm.currentState",
                          proxyDeviceDps[1] + ".:_alert_hdl.._act_state",
                          proxyDeviceDps[1] + ".agentCommunicationStatus.summary",
                          proxyDeviceDps[1] + "/Monitoring/Memory.readings.userMemory",
                          proxyDeviceDps[1] + "/Monitoring/Memory.readings.usedSwap",
                          proxyDeviceDps[1] + "/Monitoring/Cpus.readings.averageLoad.idle",
                          proxyDeviceDps[1] + "/Monitoring/Memory.readings.free:_online.._invalid",
                          sFrontEnd+".alarm",
                          sFrontEnd+".enabled");
    g_bCallbackConnected = (iRes >= 0);    
  }else{
    hostIcemonHost_WidgetDisconnection(g_sWidgetType);
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_WidgetDisconnect
/** dpDisconnect to the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, front-end device DP name
@param sFrontEnd input, front-end device name

*/
hostIcemonHost_WidgetDisconnect(string deviceName, string sFrontEnd, string fsmDp)
{
  int iRes;
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host widget IcemonHost "+deviceName, "hostIcemonHost_WidgetDisconnect", deviceName, g_bUnSystemAlarmPlc);
  if(g_bUnSystemAlarmPlc)
  {
    // if there were many dpConnect possibilities: e.g. dpConnect with or without _alert_hdl config
    // re-use the kind of dpConnect kept in a variable global to the widget set during hostIcemonHost_WidgetConnect
    // and do to the correct dpDisconnect
    dyn_string proxyDeviceDps;
    unGenericDpFunctions_getProxyDeviceList(deviceName, result, proxyDeviceDps);

    iRes = dpDisconnect("hostIcemonHost_WidgetCB", 
                          deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                          deviceName + ".statusInformation.selectedManager",
                          fsmDp + ".fsm.currentState",
                          proxyDeviceDps[1] + ".:_alert_hdl.._act_state",
                          proxyDeviceDps[1] + ".agentCommunicationStatus.summary",
                          proxyDeviceDps[1] + "/Monitoring/Memory.readings.userMemory",
                          proxyDeviceDps[1] + "/Monitoring/Memory.readings.usedSwap",
                          proxyDeviceDps[1] + "/Monitoring/Cpus.readings.averageLoad.idle",
                          proxyDeviceDps[1] + "/Monitoring/Memory.readings.free:_online.._invalid",
                          sFrontEnd+".alarm",
                          sFrontEnd+".enabled");
    g_bCallbackConnected = !(iRes >= 0);
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_WidgetCB
/** callback function on the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpLocked input, the lock DPE 
@param bLocked input, the lock state 
@param sDpSelectedManager input, the selected manager DPE 
@param sSelectedManager input, the selected manager 
@param sDp1 input, the DPE1 
@param blobData input, device data
@param sDpInvalid input, the invalid DPE 
@param bInvalid input, data validity
@param sDpFESystemIntegrityAlarmValue input, front-end device system integrity alarm DPE 
@param iFESystemIntegrityAlarmValue input, front-end device system integrity alarm value
@param sDpFESystemIntegrityAlarmEnabled input, front-end device system integrity enable DPE
@param bFESystemIntegrityAlarmEnabled input, front-end device system integrity enable value
*/
hostIcemonHost_WidgetCB(string sDpLocked, bool bLocked,
                                  string sDpSelectedManager, string sSelectedManager,
                                  string sFsmStateDp, string state, 
                                  string sAlertStateDp, string alertState, 
                                  string sReadoutStatusDp, int readout, 
                                  string sUserMemoryDp, float userMemory, 
                                  string sUsedSwapDp, float usedSwap, 
                                  string sCpuDp, float cpu,
                                  string sDpInvalid, bool bInvalid,                                  
                                  string sDpFESystemIntegrityAlarmValue, int iFESystemIntegrityAlarmValue, 
                                  string sDpFESystemIntegrityAlarmEnabled, bool bFESystemIntegrityAlarmEnabled)
{
  /** use the exec mechanism to allow multiple widget. **/
  string sFunction = "hostIcemonHost_"+g_sWidgetType+"Animation";
  string selectColor, sWarningLetter, sWarningColor;
  bool bSelectVisible;
  
  // 1. set the widget.
  unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, selectColor, bSelectVisible);

  // if bInvalid --> N letter and everything blue
  if (bInvalid){
    sWarningLetter = UN_WIDGET_TEXT_INVALID;
    sWarningColor = "unDataNotValid";
  }  
  else {
// if Front-end problem or problems-> O letter
    if((!bFESystemIntegrityAlarmEnabled) || (iFESystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value))
    {
      sWarningLetter = UN_WIDGET_TEXT_OLD_DATA;
      sWarningColor = "unDataNotValid";
    }
  }

  if(alertState == 0)
    txAlertPresence.visible = false;
  else
    txAlertPresence.visible = true;
  
  if(g_bSystemConnected)
    setMultiValue("WarningText", "text", sWarningLetter, 
                  "WarningText", "foreCol", sWarningColor,
                  "SelectArea", "foreCol", selectColor,
                  "LockBmp", "visible", bSelectVisible,
                  "WidgetArea", "visible", true);

//  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host widget IcemonHost "+sDp1, "hostIcemonHost_WidgetCB", sDp1, blobData, g_sWidgetType, 
//                               sWarningLetter, sWarningColor, g_bSystemConnected,
//                               selectColor, bSelectVisible);
  
  //FMC:
  string fmcNode = fwFMC_getNodeName(sUserMemoryDp);
  string fmcNodeDp = fwFMC_getNodeDp(fmcNode);

  if (isFunctionDefined(sFunction) && (sFunction != ""))
  {
    // set here all the common graphical element value or
    // give to the function all the variables
    execScript("main(string fmcNodeDp, string state, int readout, float userMemory, float usedSwap, float cpu, bool bSystemConnected, string sWidgetType, int iFESystemIntegrityAlarmValue, bool bFESystemIntegrityAlarmEnabled) {" + 
               sFunction + "(fmcNodeDp, state, readout, userMemory, usedSwap, cpu, bSystemConnected, sWidgetType, iFESystemIntegrityAlarmValue, bFESystemIntegrityAlarmEnabled);}", 
               makeDynString(), fmcNodeDp, state, readout, userMemory, usedSwap, cpu, g_bSystemConnected, g_sWidgetType, iFESystemIntegrityAlarmValue, bFESystemIntegrityAlarmEnabled);
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_MenuConfiguration
/** pop-up menu
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpName input, device DP name
@param sDpType input, device type
@param dsAccessOk input, the access control 
@param menuList output, pop-up menu to show, dyn_string to be given to the popupMenu function
*/
hostIcemonHost_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
  dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, UN_POPUPMENU_TREND_TEXT);
  
  unGenericObject_addSelectToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
  unGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
  unGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
  if (isFunctionDefined("icemon_addMoonActionsToMen"))
    icemon_addMoonActionsToMen(menuList);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_HandleMenu
/** handle the answer of the popup menu
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, device DP name
@param sDpType input, device type
@param menuList input, the access control 
@param iMenuAnswer input, selected menu value 
*/
hostIcemonHost_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int iMenuAnswer)
{
  string sDeviceName = unGenericDpFunctions_getAlias(deviceName);
  int iUN_POPUPMENU_MIN = UN_POPUPMENU_MIN;
  dyn_string exceptionInfo;
  dyn_string dsTemp, dsSplit;
  int len, i, iMenu;
  
  // handle unicos menu
  if(iMenuAnswer<iUN_POPUPMENU_MIN)
    unGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, iMenuAnswer, exceptionInfo);
  else if (iMenuAnswer == 333){ //LAn DB Info
    icemon_handleMenu(sDeviceName);
  }
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host rigthClick IcemonHost "+deviceName, "hostIcemonHost_HandleMenu", deviceName, sDpType, menuList, iMenuAnswer);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_SquareAnimation
/** animate the device Square widget
!!!!! function trigger by exec call, $-param and variable of the widget cannot be used, all the necessary data 
must be given to the function. The use of global var (global keyword declaration) is allowed
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param blobData input, device data
@param bSystemConnected input, connection state of the system, true=system connected/false=system not connected
@param sWidgetType input, the type of widget
@param iFESystemIntegrityAlarmValue input, front-end device system integrity alarm value
@param bFESystemIntegrityAlarmEnabled input, front-end device system integrity enable value
*/
hostIcemonHost_SquareAnimation(string fmcNodeDp, string state, int readout, float userMemory, float usedSwap, float cpu, bool bSystemConnected, string sWidgetType, int iFESystemIntegrityAlarmValue, bool bFESystemIntegrityAlarmEnabled)
{
  /*!!!!!!!! all parameters must be given: function launched by exec ***/
  // encapsulate all the setValue or function call with the following if(bSystemConnected)
  if(bSystemConnected) {
    string color;
    strreplace(fmcNodeDp, getSystemName(), "");
    fwCU_getStateColor(fmcNodeDp, state, color);   

    setMultiValue("txMemory", "text", "Mem: "+ (floor(userMemory*10.)/10.) + "%",
                  "txSwap", "text", "Swap: "+ (floor(usedSwap*10.)/10.) + "%",
                  "txCpu", "text", "CPU: " + floor((100.-cpu)) + "%",
                  "txState", "text", state,
                  "shStateColor", "backCol", color);
  }
//  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host widget IcemonHost Square", "hostIcemonHost_SquareAnimation", sWidgetType, blobData, bSystemConnected, 
//                               iFESystemIntegrityAlarmValue, bFESystemIntegrityAlarmEnabled);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_SquareDisconnection
/** animate the the device Square widget when the system is disconnected 
!!!!! function trigger by exec call, $-param and variable of the widget cannot be used, all the necessary data 
must be given to the function. The use of global var (global keyword declaration) is allowed

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sWidgetType input, the type of widget
*/
hostIcemonHost_SquareDisconnection(string sWidgetType)
{
  // set all the graphical element of the widget
  setMultiValue("Body1", "backCol", "unDataNoAccess",
                "WarningText", "text", "",
                "WidgetArea", "visible", false,
                "SelectArea", "foreCol", "",
                "LockBmp", "visible", false);
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host widget IcemonHost Square", "hostIcemonHost_SquareDisconnection", sWidgetType);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_WidgetRegisterAlertCB
/** widget DistributedControl callback
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDp input, the DistributedControl system name DP name
@param bSystemConnected input, the state of the system name
*/
hostIcemonHost_WidgetRegisterAlertCB(string sDp, bool bSystemConnected)
{
  string deviceName, sSystemName;
  int iAction, iRes;
  dyn_string exceptionInfo;
  string sFrontEnd;
  bool bRemote;
  
  deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
  if (deviceName == "")  // In case of disconnection
  {
    deviceName = g_sDpName;
  }
  g_bUnSystemAlarmPlc = false;
// get the systemName
  sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
  sFrontEnd = unGenericObject_GetPLCNameFromDpName(deviceName);

  unDistributedControl_isRemote(bRemote, sSystemName);
  if(bRemote)
    g_bSystemConnected = bSystemConnected;
  else
    g_bSystemConnected = true;

  if(sFrontEnd != ""){
    sFrontEnd = sSystemName+c_unSystemAlarm_dpPattern+DS_pattern+sFrontEnd;
//DebugN(sSystemName, sFrontEnd);
    if(dpExists(sFrontEnd)) {
      g_bUnSystemAlarmPlc = true;
    }
  }
  g_sDeviceName = deviceName;
  g_sFrontEnd = sFrontEnd;

  unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host widget IcemonHost "+deviceName, "hostIcemonHost_WidgetRegisterCB", sDp, deviceName, g_bCallbackConnected, g_bDisconnectCB, iAction);
  switch (iAction)
  {
    case UN_ACTION_DISCONNECT:
      hostIcemonHost_WidgetDisconnection(g_sWidgetType);
      break;
    case UN_ACTION_DPCONNECT:
      g_sDpName = deviceName;
      if(g_bUnSystemAlarmPlc) {
        iRes = dpConnect("hostIcemonHost_WidgetLockCallBack", deviceName + ".ProcessInput.data:_lock._alert_hdl._locked");
        g_bCallbackConnected = (iRes >= 0);
      }
      else
        hostIcemonHost_WidgetDisconnection(g_sWidgetType);
      break;
    case UN_ACTION_DPDISCONNECT_DISCONNECT:
      if(g_bUnSystemAlarmPlc) {
        iRes = dpDisconnect("hostIcemonHost_WidgetLockCallBack", deviceName + ".ProcessInput.data:_lock._alert_hdl._locked");
        g_bCallbackConnected = !(iRes >= 0);
      }
      if(g_bDisconnectCB)
        hostIcemonHost_WidgetDisconnectAlert(deviceName, sFrontEnd);
      hostIcemonHost_WidgetDisconnection(g_sWidgetType);
      break;
    case UN_ACTION_NOTHING:
    default:
      break;
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_WidgetLockCallBack
/** dpConnect to the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpe input, device DPE name
@param bLockState input, state of the alert lock
*/
hostIcemonHost_WidgetLockCallBack(string sDpe, bool bLockState)
{
  int iRes;

  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host widget IcemonHost "+g_sDeviceName, "hostIcemonHost_WidgetLockCallBack", sDpe, bLockState, g_bDisconnectCB, g_sDeviceName, g_sFrontEnd);
  if(bLockState) { // lock set so disconnect if not already disconnected
// no need to check if it was already set before because it is not possible to do 2 dpSet on the lock the Ev reject the message
    if(g_bDisconnectCB) {
      hostIcemonHost_WidgetDisconnectAlert(g_sDeviceName, g_sFrontEnd);
    }
    hostIcemonHost_WidgetDisconnection(g_sWidgetType);
    // set the WidgetArea to true, so that even if there is a problem with the lock, one can do a rigth click and correct the lock state
    setValue("WidgetArea", "visible", true);
  }
  else { // lock not set so connect if not already connected
// no need to check if it was already set before because it is not possible to do 2 dpSet on the lock the Ev reject the message
    if(!g_bDisconnectCB) {
      hostIcemonHost_WidgetConnectAlert(g_sDeviceName, g_sFrontEnd);
    }
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_WidgetConnectAlert
/** dpConnect to the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, front-end device DP name
@param sFrontEnd input, front-end device name
*/
hostIcemonHost_WidgetConnectAlert(string deviceName, string sFrontEnd)
{
  int iRes;
  string sSystemName;
  
//  check if the device name is of type "IcemonHost" if not like no front-end. -> not available.
  if(dpTypeName(deviceName) != "IcemonHost")
    g_bUnSystemAlarmPlc = false;
  
  sSystemName = unGenericDpFunctions_getSystemName(deviceName);
  
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host widget IcemonHost "+deviceName, "hostIcemonHost_WidgetConnectAlert", deviceName, g_bUnSystemAlarmPlc);
  if(g_bUnSystemAlarmPlc)
  {
    // set all widget global variable usefull for the callback, e.g.: description, format, unit, etc.
    
    // if there are many dpConnect possibilities: e.g. dpConnect with or without _alert_hdl config
    // keep the kind of dpConnect in a variable global to the widget and do a swith for the dpConnect
    // and do to the correct dpDisconnect in the hostIcemonHost_WidgetDisconnectAlert
    iRes = dpConnect("hostIcemonHost_WidgetCB", 
                          deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                          deviceName + ".statusInformation.selectedManager",
                          deviceName + ".ProcessInput.data",
                          deviceName + ".ProcessInput.data:_online.._invalid",
                          sFrontEnd+".alarm",
                          sFrontEnd+".enabled");
    g_bDisconnectCB = (iRes >= 0);    
  }else{
    hostIcemonHost_WidgetDisconnection(g_sWidgetType);
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_WidgetDisconnectAlert
/** dpDisconnect to the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, front-end device DP name
@param sFrontEnd input, front-end device name

*/
hostIcemonHost_WidgetDisconnectAlert(string deviceName, string sFrontEnd)
{
  int iRes;

  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host widget IcemonHost "+deviceName, "hostIcemonHost_WidgetDisconnectAlert", deviceName, g_bUnSystemAlarmPlc);
  if(g_bUnSystemAlarmPlc)
  {
    // if there were many dpConnect possibilities: e.g. dpConnect with or without _alert_hdl config
    // re-use the kind of dpConnect kept in a variable global to the widget set during hostIcemonHost_WidgetConnectAlert
    // and do to the correct dpDisconnect
    iRes = dpDisconnect("hostIcemonHost_WidgetCB", 
                          deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                          deviceName + ".statusInformation.selectedManager",
                          deviceName + ".ProcessInput.data",
                          deviceName + ".ProcessInput.data:_online.._invalid",
                          sFrontEnd+".alarm",
                          sFrontEnd+".enabled");
    g_bDisconnectCB = !(iRes >= 0);
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_FaceplateStatusRegisterAlertCB
/** faceplate DistributedControl callback of the faceplate status panel
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDp input, the DistributedControl system name DP name
@param bSystemConnected input, the state of the system name
*/
hostIcemonHost_FaceplateStatusRegisterAlertCB(string sDp, bool bSystemConnected)
{
  string deviceName, sSystemName;
  int iAction, iRes;
  dyn_string exceptionInfo;
  bool bRemote;
  
  deviceName = unGenericDpFunctions_getDpName($sDpName);
  sSystemName = unGenericDpFunctions_getSystemName(deviceName);
  unDistributedControl_isRemote(bRemote, sSystemName);
  if(bRemote)
    g_bSystemConnected = bSystemConnected;
  else
    g_bSystemConnected = true;
  
  unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host faceplate IcemonHost "+sDp, "hostIcemonHost_FaceplateStatusRegisterAlertCB", sDp, deviceName, g_bCallbackConnected, g_bDisconnectCB, iAction);
  switch (iAction)
  {
    case UN_ACTION_DISCONNECT:
      hostIcemonHost_FaceplateStatusDisconnection();
      break;
    case UN_ACTION_DPCONNECT:
      hostIcemonHost_FaceplateConnectAlert(deviceName);
      break;
    case UN_ACTION_DPDISCONNECT_DISCONNECT:
      hostIcemonHost_FaceplateDisconnectAlert(deviceName);
      hostIcemonHost_FaceplateStatusDisconnection();
      break;
    case UN_ACTION_NOTHING:
    default:
      break;
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_FaceplateConnectAlert
/** dpConnect to the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
*/
hostIcemonHost_FaceplateConnectAlert(string deviceName)
{
  int iAction, iRes;
  dyn_string exceptionInfo;
  string sFrontEnd, sSystemName;
  
  g_sDeviceAlias = unGenericDpFunctions_getAlias(deviceName);

  g_bUnSystemAlarmPlc = false;
// get the systemName
  sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
  sFrontEnd = unGenericObject_GetPLCNameFromDpName(deviceName);
  g_sFrontEndDp = sSystemName+sFrontEnd;
  g_sFrontEndAlias = unGenericDpFunctions_getAlias(g_sFrontEndDp);
  
  if(sFrontEnd != ""){
    sFrontEnd = sSystemName+c_unSystemAlarm_dpPattern+DS_pattern+sFrontEnd;
//DebugN(sSystemName, sFrontEnd);
    if(dpExists(sFrontEnd)) {
      g_bUnSystemAlarmPlc = true;
    }
  }
  g_sDeviceName = deviceName;
  g_sFrontEnd = sFrontEnd;
    
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host faceplate IcemonHost "+deviceName, "hostIcemonHost_FaceplateConnectAlert", deviceName, sFrontEnd, g_bUnSystemAlarmPlc, g_bDisconnectCB);
  if(g_bUnSystemAlarmPlc) { 
    iRes = dpConnect("hostIcemonHost_FaceplateLockCallBack", deviceName + ".ProcessInput.data:_lock._alert_hdl._locked");
    g_bCallbackConnected = (iRes >= 0);
  }
  else
    hostIcemonHost_FaceplateStatusDisconnection();
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_FaceplateDisconnectAlert
/** dpDisconnect to the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
*/
hostIcemonHost_FaceplateDisconnectAlert(string deviceName)
{
  int iRes;
  string sFrontEnd, sSystemName;
  
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host faceplate IcemonHost "+deviceName, "hostIcemonHost_FaceplateDisconnectAlert", deviceName, g_bUnSystemAlarmPlc, g_bDisconnectCB);
  if(g_bUnSystemAlarmPlc) {
// get the systemName
    sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
    sFrontEnd = unGenericObject_GetPLCNameFromDpName(deviceName);
    sFrontEnd = sSystemName+c_unSystemAlarm_dpPattern+DS_pattern+sFrontEnd;

    iRes = dpDisconnect("hostIcemonHost_FaceplateLockCallBack", deviceName + ".ProcessInput.data:_lock._alert_hdl._locked");
    g_bCallbackConnected = !(iRes >= 0);
    if(g_bDisconnectCB) {
      // if there were many dpConnect possibilities: e.g. dpConnect with or without _alert_hdl config
      // re-use the kind of dpConnect kept in a variable global to the faceplate set during hostIcemonHost_FaceplateConnect
      // and do to the correct dpDisconnect
      iRes = dpDisconnect("hostIcemonHost_FaceplateStatusAnimationCB",
                        deviceName + ".ProcessInput.data",
                        deviceName + ".ProcessInput.data:_online.._invalid",
                        deviceName + ".ProcessInput.iData",
                        sFrontEnd+".alarm",
                        sFrontEnd+".enabled");
      g_bDisconnectCB = !(iRes >= 0);
    }
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// hostIcemonHost_FaceplateLockCallBack
/** dpConnect to the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpe input, device DPE name
@param bLockState input, state of the alert lock
*/
hostIcemonHost_FaceplateLockCallBack(string sDpe, bool bLockState)
{
  int iRes;

  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host faceplate IcemonHost "+g_sDeviceName, "hostIcemonHost_FaceplateLockCallBack", sDpe, bLockState, g_bDisconnectCB, g_sDeviceName, g_sFrontEnd);
  if(bLockState) { // lock set so disconnect if not already disconnected
// no need to check if it was already set before because it is not possible to do 2 dpSet on the lock the Ev reject the message
    if(g_bDisconnectCB) {
      iRes = dpDisconnect("hostIcemonHost_FaceplateStatusAnimationCB",
                      g_sDeviceName + ".ProcessInput.data",
                      g_sDeviceName + ".ProcessInput.data:_online.._invalid",
                      g_sDeviceName + ".ProcessInput.iData",
                      g_sFrontEnd+".alarm",
                      g_sFrontEnd+".enabled");
      g_bDisconnectCB = !(iRes >= 0);
    }
    hostIcemonHost_FaceplateStatusDisconnection();
  }
  else { // lock not set so connect if not already connected
// no need to check if it was already set before because it is not possible to do 2 dpSet on the lock the Ev reject the message
    if(!g_bDisconnectCB) {
      // set all faceplate global variable usefull for the callback, e.g.: description, format, unit, etc.
      // set graphical element of the faceplate to the init state if needed
    
      // if there are many dpConnect possibilities: e.g. dpConnect with or without _alert_hdl config
      // keep the kind of dpConnect in a variable global to the faceplate and do a swith for the dpConnect
      // and do to the correct dpDisconnect in the hostIcemonHost_FaceplateDisconnect
      iRes = dpConnect("hostIcemonHost_FaceplateStatusAnimationCB",
                        g_sDeviceName + ".ProcessInput.data",
                        g_sDeviceName + ".ProcessInput.data:_online.._invalid",
                        g_sDeviceName + ".ProcessInput.iData",
                        g_sFrontEnd+".alarm",
                        g_sFrontEnd+".enabled");
      g_bDisconnectCB = (iRes >= 0);
    }
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

//@}
