/**@file

// hostSystemIntegrity_IcemonHostsFE.ctl
This library contains the system integrity functions of the IcemonHostsFE.

@par Creation Date
  dd/mm/yyyy

@par Modification History
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  CTRL

@author 
  the author (DEP-GROUP)
*/

//@{
// global declaration
global dyn_string g_hostSystemIntegrity_IcemonHostsFE_DSList; // list of the IcemonHostsFE-DS dp checked
global dyn_int g_hostSystemIntegrity_IcemonHostsFE_DSThreadId; // list of the IcemonHostsFE-DS thread Id checking the counter
global int g_hostSystemIntegrity_IcemonHostsFECheckingDelay;
//@}

//@{

//------------------------------------------------------------------------------------------------------------------------
// hostSystemIntegrity_IcemonHostsFE_Initialize
/** Get the list of IcemonHostsFE checked and enabled by the systemIntegrity. 
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  CTRL

@param dsResult output, list of enabled front-end systemIntegrity
*/
hostSystemIntegrity_IcemonHostsFE_Initialize(dyn_string &dsResult)
{
  dyn_string dsList;
  int len, i;
  string dpToCheck;
  bool enabled;

  dpGet(UN_SYSTEM_INTEGRITY_ICEMONHOSTSFE+UN_SYSTEMINTEGRITY_EXTENSION+".config.data", dsList);
  hostSystemIntegrity_IcemonHostsFE_DataCallback("", dsList);

  dsList = dpNames(c_unSystemAlarm_dpPattern+DS_pattern+"*", c_unSystemAlarm_dpType);
  len = dynlen(dsList);
  for(i = 1; i<=len; i++) {
    dpToCheck = dpSubStr(dsList[i], DPSUB_DP);
    dpToCheck = substr(dpToCheck, strlen(c_unSystemAlarm_dpPattern+DS_pattern),strlen(dpToCheck));
    if(dpExists(dpToCheck)) {
      if(dpTypeName(dpToCheck) == HOST_ICEMONHOSTSFE_DPTYPE) {
        dpGet(dsList[i]+".enabled", enabled);
        if(enabled)
          dynAppend(dsResult, dpToCheck);
      }
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------
// hostSystemIntegrity_IcemonHostsFE_DataCallback
/** Callback on the configuration data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  CTRL

@param sDpe1  input, dpe
@param dsConfigData input, configuration data
*/
hostSystemIntegrity_IcemonHostsFE_DataCallback(string sDpe1, dyn_string dsConfigData)
{
  dyn_string dsTemp=dsConfigData;

  // check the size of the configuration data and add default value if some data are missing
  while(dynlen(dsTemp) <=1)
    dynAppend(dsTemp, "0");

  g_hostSystemIntegrity_IcemonHostsFECheckingDelay = dsTemp[1];
  if(g_hostSystemIntegrity_IcemonHostsFECheckingDelay <= 0)
    g_hostSystemIntegrity_IcemonHostsFECheckingDelay = c_hostSystemIntegrity_defaultICEMONHOSTSFECheckingDelay;
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host systemIntegrity", "hostSystemIntegrity_IcemonHostsFE_DataCallback", dsConfigData, g_hostSystemIntegrity_IcemonHostsFECheckingDelay);
}

//------------------------------------------------------------------------------------------------------------------------
// hostSystemIntegrity_IcemonHostsFE_HandleCommand
/** Callback on the systemIntegrity command.
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  CTRL

@param sDpe1  input, dpe
@param command input, type of command
@param sDpe2  input, dpe
@param parameters input, parameters of the command
*/
hostSystemIntegrity_IcemonHostsFE_HandleCommand(string sDpe1, int command, string sDpe2, dyn_string parameters)
{
  dyn_string exceptionInfo;
  int i, len =dynlen(parameters);
  dyn_string dsResult;
   string sResult;
 
  switch(command) {
    case UN_SYSTEMINTEGRITY_ADD: // add the _UnSystemAlarm DP
      for(i=1; i<=len; i++) {
        hostSystemIntegrity_IcemonHostsFE_checking(parameters[i], true, true, exceptionInfo);
      }
      break;
    case UN_SYSTEMINTEGRITY_DELETE: // delete the _UnSystemAlarm DP
      for(i=1; i<=len; i++) {
        hostSystemIntegrity_IcemonHostsFE_checking(parameters[i], false, false, exceptionInfo);
        // remove all _UnSystemAlarm dp from the alert list of applicationDP if it is in
        _unSystemIntegrity_modifyApplicationAlertList(c_unSystemAlarm_dpPattern+DS_pattern+parameters[i], false, exceptionInfo);
        // delete all the _UnSystemAlarm dps
        if(dpExists(c_unSystemAlarm_dpPattern+DS_pattern+parameters[i]))
          dpDelete(c_unSystemAlarm_dpPattern+DS_pattern+parameters[i]);
      }
      break;
    case UN_SYSTEMINTEGRITY_ENABLE: // enable the _UnSystemAlarm DP
      for(i=1; i<=len; i++) {
        if(dpExists(c_unSystemAlarm_dpPattern+DS_pattern+parameters[i]))
          hostSystemIntegrity_IcemonHostsFE_checking(parameters[i], false, true, exceptionInfo);
      }
      break;
    case UN_SYSTEMINTEGRITY_DISABLE: // disable the _UnSystemAlarm DP
      for(i=1; i<=len; i++) {
        if(dpExists(c_unSystemAlarm_dpPattern+DS_pattern+parameters[i]))
          hostSystemIntegrity_IcemonHostsFE_checking(parameters[i], false, false, exceptionInfo);
      }
      break;
    case UN_SYSTEMINTEGRITY_DIAGNOSTIC: // give the list of _UnSystemAlarm DP and the state
      dpSet(UN_SYSTEM_INTEGRITY_ICEMONHOSTSFE+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.commandResult", command, 
          UN_SYSTEM_INTEGRITY_ICEMONHOSTSFE+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.result", g_hostSystemIntegrity_IcemonHostsFE_DSList);
      break;
    case UN_SYSTEMINTEGRITY_EXTENDED_DIAGNOSTIC: // extended diagnostic
      dynAppend(dsResult, getCurrentTime());
      dynAppend(dsResult, "g_hostSystemIntegrity_IcemonHostsFE_DSList:");
      len = dynlen(g_hostSystemIntegrity_IcemonHostsFE_DSList);
      for(i=1;i<=len;i++) {
        sResult = g_hostSystemIntegrity_IcemonHostsFE_DSList[i];
        dynAppend(dsResult, sResult);
      }
      dynAppend(dsResult, "g_hostSystemIntegrity_IcemonHostsFE_DSThreadId:");
      len = dynlen(g_hostSystemIntegrity_IcemonHostsFE_DSThreadId);
      for(i=1;i<=len;i++) {
        sResult = g_hostSystemIntegrity_IcemonHostsFE_DSThreadId[i];
        dynAppend(dsResult, sResult);
      }
      dynAppend(dsResult, "g_hostSystemIntegrity_IcemonHostsFECheckingDelay:" + g_hostSystemIntegrity_IcemonHostsFECheckingDelay);
      dpSet(UN_SYSTEM_INTEGRITY_ICEMONHOSTSFE+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.commandResult", command, 
          UN_SYSTEM_INTEGRITY_ICEMONHOSTSFE+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.result", dsResult);
      DebugTN(dsResult);
      break;
    case UN_SYSTEMINTEGRITY_DEBUG:
      if(dynlen(parameters) >= 2)
      {
        g_b32DebugLevel = parameters[1];
        g_sDebugFilter = parameters[2];
      }
      else
      {
        g_b32DebugLevel = 0;
        g_sDebugFilter = "";
      }
      break;
    default:
      break;
  }

  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host systemIntegrity", "hostSystemIntegrity_IcemonHostsFE_HandleCommand", command, parameters);
  if(dynlen(exceptionInfo) > 0) {
    if(isFunctionDefined("unMessageText_sendException")) {
      unMessageText_sendException("*", "*", "hostSystemIntegrity_IcemonHostsFE_HandleCommand", "user", "*", exceptionInfo);
    }
// handle any error uin case the send message failed
    if(dynlen(exceptionInfo) > 0) {
      DebugN(getCurrentTime(), exceptionInfo);
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------
// hostSystemIntegrity_IcemonHostsFE_checking
/** This function register/de-register the callback funtion of IcemonHostsFE manager and creates the _unSystemAlarm_IcemonHostsFE dp 
with the alarm config but it does not delete it.

  
@par Constraints
  None

@par Usage
  Internal

@par PVSS managers
  CTRL

@param sDp  input, data point name
@param bCreate  input, true create the dp and the alarm config if it does not exist, enable it
@param bRegister  input, true do a dpConnect, false do a dpDisconnect
@param exceptionInfo output, exception are returned here
*/
hostSystemIntegrity_IcemonHostsFE_checking(string sDp, bool bCreate, bool bRegister, dyn_string &exceptionInfo)
{
  string sDpToChek, dp, dpPlc, plcHostname, description;
  int res, thId, pos, drvNum, len,i;
  bool bError = false;
  dyn_string plc_plc_list;

  string alertPanel,alertHelp;
  bool configExists, isActive;
  int alertConfigType;
  dyn_string alertTexts,alertClasses,summaryDpeList,alertPanelParameters;
  dyn_float alertLimits;
  
// remove the system name
  res = strpos(sDp, ":");
  if(res >= 0)
    dp = substr(sDp, res+1, strlen(sDp));
  else
    dp = sDp;
  sDpToChek = dp;

  dp = c_unSystemAlarm_dpPattern+DS_pattern+dp;

// in case the Dp is not of type _UnPlc do nothing. Should not happen, just for security.
  if(dpTypeName(sDpToChek) != HOST_ICEMONHOSTSFE_DPTYPE)
    return;
    
  dpGet(sDpToChek+".communication.driver_num", drvNum);

  if(bCreate) {
// get the plc hostname and the driver number
    plcHostname = unGenericDpFunctions_getAlias(sDpToChek);
// create all the _UnSystemIntegrity Dp and its alarm config if it is not existing
    if(!dpExists(dp)) {
      description = getCatStr("unSystemIntegrity", "PLC_DS_DESCRIPTION")+plcHostname+" -> DS driver "+ drvNum;
      unSystemIntegrity_createSystemAlarm(sDpToChek, DS_pattern, description, exceptionInfo);
    }
  }

  if(bRegister) {
// connect to the callback function if not already done
    pos = _unSystemIntegrity_isInList(g_hostSystemIntegrity_IcemonHostsFE_DSList, dp);
    if(pos <= 0) {
  // add it in the list, because one of the callback function needs it.        
      pos = _unSystemIntegrity_setList(g_hostSystemIntegrity_IcemonHostsFE_DSList, dp, true);

// start the thread for checking the counter
      thId = startThread(UN_SYSTEM_INTEGRITY_ICEMONHOSTSFE_check, sDpToChek, drvNum);
      g_hostSystemIntegrity_IcemonHostsFE_DSThreadId[pos] = thId;
      if(thId<0)
        bError = true;

      if(!bError) {
        // do here any dpConnect to the front-end DPE, or proxy DPE
        // keep the DPs in global list to be able to dpDisconnect it later
      }
      if(bError) {
        pos = _unSystemIntegrity_isInList(g_hostSystemIntegrity_IcemonHostsFE_DSList, dp);
        if(pos > 0) {
    // kill the threads if they were started
          res = stopThread(g_hostSystemIntegrity_IcemonHostsFE_DSThreadId[pos]);
// and then remove it from the list
          pos = _unSystemIntegrity_setList(g_hostSystemIntegrity_IcemonHostsFE_DSList, dp, false);
//DebugN("rmv", res);
          dynRemove(g_hostSystemIntegrity_IcemonHostsFE_DSThreadId, pos);
        }

        fwException_raise(exceptionInfo, "ERROR", 
              "hostSystemIntegrity_IcemonHostsFE_checking():"+getCatStr("unSystemIntegrity", "CANNOT_DP_CONNECT") +dp,"");
      }
      else {
//DebugN("Enable", g_hostSystemIntegrity_IcemonHostsFE_DSThreadId, "end");
      // set the enable to true and activate the alarm if it is not activated of all the _UnSystemIntegrity DP.
        dpSet(dp+".enabled", true);
        unAlarmConfig_mask(dp+".alarm", false, exceptionInfo);
      // get all the PLC-PLC dp, and enable them
        plc_plc_list = dpNames(c_unSystemAlarm_dpPattern+PLC_PLC_pattern+sDpToChek+"*", c_unSystemAlarm_dpType);
        len = dynlen(plc_plc_list);
        for(i=1;i<=len;i++) {
          dpSet(plc_plc_list[i]+".enabled", true);
          unAlarmConfig_mask(plc_plc_list[i]+".alarm", false, exceptionInfo);
        }

      // get all the FESystemAlarm dp, and enable them, check the Ok state and set the correct value
        plc_plc_list = dpNames(c_unSystemAlarm_dpPattern+FE_pattern+sDpToChek+"*", c_unSystemAlarm_dpType);
        len = dynlen(plc_plc_list);
        for(i=1;i<=len;i++) {
          dpSet(plc_plc_list[i]+".enabled", true);
          unAlarmConfig_mask(plc_plc_list[i]+".alarm", false, exceptionInfo);
        }

// set the GQ bit so all data will bypass the smoothing
        dpSet("_Driver" + drvNum + ".SM:_original.._value", 1);
        dpSet("_Driver" + drvNum + ".GQ:_original.._value", 1);
      }
    }
    unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host systemIntegrity", "hostSystemIntegrity_IcemonHostsFE_checking register", sDp,
                                 g_hostSystemIntegrity_IcemonHostsFE_DSThreadId, g_hostSystemIntegrity_IcemonHostsFE_DSThreadId);
  }
  else {
// disconnect the callback function
    pos = _unSystemIntegrity_isInList(g_hostSystemIntegrity_IcemonHostsFE_DSList, dp);
    if(pos > 0) {
// kill the thread
      res = stopThread(g_hostSystemIntegrity_IcemonHostsFE_DSThreadId[pos]);
      if(res<0)
        bError = true;

        // do here any dpDisconnect to the front-end DPE, or proxy DPE
        // use the global list set during the dpConnect 

      if(bError) {
        fwException_raise(exceptionInfo, "ERROR", 
              "hostSystemIntegrity_IcemonHostsFE_checking():"+getCatStr("unSystemIntegrity", "CANNOT_DP_DISCONNECT") +dp,"");
      }
      else {
      // remove from list
        pos = _unSystemIntegrity_setList(g_hostSystemIntegrity_IcemonHostsFE_DSList, dp, false);
//DebugN("rmv", res);
        dynRemove(g_hostSystemIntegrity_IcemonHostsFE_DSThreadId, pos);
      // set the enable to false and de-activate the alarm and acknowledge it if necessary of all the _UnSystemIntegrity DP.
        dpSet(dp+".enabled", false, dp+".alarm", c_unSystemIntegrity_no_alarm_value);
        unAlarmConfig_mask(dp+".alarm", true, exceptionInfo);
      // get all the PLC-PLC dp, and disable them
        plc_plc_list = dpNames(c_unSystemAlarm_dpPattern+PLC_PLC_pattern+sDpToChek+"*", c_unSystemAlarm_dpType);
        len = dynlen(plc_plc_list);
        for(i=1;i<=len;i++) {
// do not reset the alarm value because this DPE should be connected to an address config
          dpSet(plc_plc_list[i]+".enabled", false);
          unAlarmConfig_mask(plc_plc_list[i]+".alarm", true, exceptionInfo);
        }

      // get all the FESystemAlarm dp, and disable them, check the Ok state and set the correct value
        plc_plc_list = dpNames(c_unSystemAlarm_dpPattern+FE_pattern+sDpToChek+"*", c_unSystemAlarm_dpType);
        len = dynlen(plc_plc_list);
        for(i=1;i<=len;i++) {
// do not reset the alarm value because this DPE should be connected to an address config
          dpSet(plc_plc_list[i]+".enabled", false);
          unAlarmConfig_mask(plc_plc_list[i]+".alarm", true, exceptionInfo);
        }
      }
    }
    unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host systemIntegrity", "hostSystemIntegrity_IcemonHostsFE_checking deregister", sDp, 
                                 g_hostSystemIntegrity_IcemonHostsFE_DSList, g_hostSystemIntegrity_IcemonHostsFE_DSThreadId);
  }
}

//------------------------------------------------------------------------------------------------------------------------
// hostSystemIntegrity_IcemonHostsFECheck
/** This function checks if the counter of the IcemonHostsFE dp is modified periodically. 
A _UnSystemAlarm is set if the counter is not updated periodically.

@par Constraints
  None

@par Usage
  Internal

@par PVSS managers
  CTRL

@param sFeDp  input, data point name
@param iDrvNum  input, the driver number
*/
hostSystemIntegrity_IcemonHostsFECheck(string sFeDp, int iDrvNum)
{
  int waitingTime, oldAlarmValue = -1, oldTimingAlarm = -1;
  int oldValue = -1, newValue, alarmValue, iTime;
  bool bEnabled;
  string sModPlc;
  int res, iPlcNumber;
  
  //get front end proxies:
  dyn_string proxyDps;
  unGenericDpFunctions_getProxyDeviceList(sFeDp, res, proxyDps);
  
  while(true) {
  // if 0 set it to g_hostSystemIntegrity_IcemonHostsFECheckingDelay
    waitingTime = g_hostSystemIntegrity_IcemonHostsFECheckingDelay;
    if(waitingTime <= 0)
      waitingTime = c_hostSystemIntegrity_defaultICEMONHOSTSFECheckingDelay;
  // wait
    delay(waitingTime);

    dyn_int states;
    dpGet(proxyDps, states);
    if(dynSum(states) == dynlen(states)) //All necessary DIM API managers are running
    {
      alarmValue = c_unSystemIntegrity_no_alarm_value;
    }
    else
    {
      alarmValue = c_unSystemIntegrity_alarm_value_level1;
    }

    unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "host systemIntegrity "+sFeDp, "hostSystemIntegrity_IcemonHostsFECheck check", sFeDp, 
                                 waitingTime, oldValue, newValue, oldAlarmValue, alarmValue);
    
    if(oldAlarmValue != alarmValue) {
      dpSet(c_unSystemAlarm_dpPattern+DS_pattern+sFeDp+".alarm", alarmValue);
      // set the GQ bit so all data will bypass the smoothing
//      dpSet("_Driver" + iDrvNum + ".SM:_original.._value", 1);
//      dpSet("_Driver" + iDrvNum + ".GQ:_original.._value", 1);
    }

    oldValue = newValue;
    oldAlarmValue = alarmValue;
  }
}

//------------------------------------------------------------------------------------------------------------------------

//@}
